/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors.svn;


import it.eng.configurazione.bean.ExtractionProcess;
import it.eng.configurazione.bean.Field;
import it.eng.configurazione.bean.Operation;
import it.eng.configurazione.bean.OperationParameter;
import it.eng.extractors.AbstractExtractor;
import it.eng.extractors.ExtratorInterface;
import it.eng.extractors.GenericItem;
import it.eng.extractors.GenericItemInterface;
import it.eng.extractors.GroovyExecutor;

import java.io.ByteArrayOutputStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tmatesoft.svn.core.SVNDirEntry;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.io.SVNRepository;

public class SvnExtractor extends AbstractExtractor implements ExtratorInterface {

	Log log = LogFactory.getLog(SvnExtractor.class);

	public static final String URL = "URL";
	public static final String USER = "USER";
	public static final String PASSWORD = "PASSWORD";

	private int metricNumber=0;
	private int actualMetric=0;
	private ArrayList<Map<String, Object>> workList = null;

	private int livelli = -1;
	private long revision = -1;

	private SVNRepository repository = null;
	private SvnBLogic svnBL = null;
	private MetricsJava metricsJava = new MetricsJava();

	public SvnExtractor(){
	}

	public void init(ExtractionProcess extractionProcess) {

		Iterator<Operation> iter=extractionProcess.getOperation().iterator();
		operazione=iter.next();

		String url = operazione.getDataSource().readConnetionParameterValue(URL);
		String name = operazione.getDataSource().readConnetionParameterValue(USER);
		String password = operazione.getDataSource().readConnetionHideParameterValue(PASSWORD);
		idDataSource = new Integer(operazione.getDataSource().getId());
		String idProject = "";


		ArrayList<String> listMetrics = new ArrayList<String>(); 
		String urlOp = "";
		Iterator iterOp = operazione.getParameter().iterator();
		while (iterOp.hasNext()) {
			OperationParameter oP = (OperationParameter)iterOp.next();
			String paramName = oP.getName().substring(oP.getName().lastIndexOf(".") + 1);
			idProject = oP.getName().substring(0,oP.getName().lastIndexOf("."));

			if ("url".equals(paramName)) {
				if (!"".equals(oP.getValue())) {
					urlOp = oP.getValue();
				}
			} else if ("user".equals(paramName)) {
				if (!"".equals(oP.getValue())) {
					name = oP.getValue(); 
				}
			} else if ("pwd".equals(paramName)) {
				if (!"".equals(oP.getValue())) {
					password = oP.getValue();
				}
			} else if ("liv".equals(paramName)) {
				if (!"".equals(oP.getValue())) {
					livelli = Integer.valueOf(oP.getValue());
				}
			} else if ("metrics".equals(paramName)) {
				listMetrics.add(oP.getValue());
			}
			
		}

		
		Collection<SVNDirEntry> fileCollection = new HashSet<SVNDirEntry>();
		try {
			svnBL = new SvnBLogic();
			repository = svnBL.getRepository(url, name, password);
			svnBL.getListEntries(fileCollection, repository, "java", urlOp, revision, livelli);
		} catch (SVNException svnE) {
			svnE.printStackTrace();
		}

		
		workList = new ArrayList<Map<String, Object>>();
		// genero insieme file e metriche
		Iterator<SVNDirEntry> iterFile = fileCollection.iterator();
		while (iterFile.hasNext()) {
			SVNDirEntry entry = iterFile.next();
			try {
				String filePath = entry.getURL().getPath();
				filePath=filePath.substring(1, filePath.length());

				ByteArrayOutputStream baos = svnBL.getFile(repository, filePath, revision);
				
				Iterator<String> iterMetrics = listMetrics.iterator();
				while (iterMetrics.hasNext()) {
					String metrica  = iterMetrics.next();
					Object[] args = { baos };

					Map<String, Object> workitemFields = new HashMap<String, Object>();
					workitemFields.put("ID_PROJECT", getValueFromField("ID_PROJECT",idProject));
					workitemFields.put("PACKAGE_NAME",getValueFromField("PACKAGE_NAME",metricsJava.execMethod(metricsJava, "getPACKAGE_NAME",args)));
					workitemFields.put("SOURCE_NAME",getValueFromField("SOURCE_NAME",entry.getName()) );
//					 calcolo metriche in base a operation_field
					Object value = metricsJava.execMethod(metricsJava, "get"+metrica,args);
					
					workitemFields.put("ID_METRIC", getValueFromField("ID_METRIC",metrica));
					workitemFields.put("METRIC_VALUE", getValueFromField("METRIC_VALUE",value));
					workitemFields.put("ID_TOOL", getValueFromField("ID_TOOL",idDataSource));
					workList.add(workitemFields);
				}

			} catch (SVNException svne) {
				log.error("error while fetching the file contents and properties: " + svne.getMessage(), svne);
			}
		}
		metricNumber=workList.size();
	
	}

	private Object getValueFromField(String fieldName, Object value) {
		Object retVal = value;
		Iterator<Field> iterField = operazione.getField().iterator();	// elenco metriche da estrarre
		while (iterField.hasNext()) {
			Field field = iterField.next();
			if (retVal!=null) {
				if (field.getName().equals(fieldName)) {
					String type = field.getType();
					if ("String".equalsIgnoreCase(type)){
						retVal= retVal.toString();
						if (field.getMappingScript()!=null && !"".equals(field.getMappingScript())){
							try{
								retVal = GroovyExecutor.execute(retVal,field.getMappingScript());
							}catch (Throwable e  ){
								e.printStackTrace();
							}
						}
					} else if ("Integer".equalsIgnoreCase(type)){
						retVal= new Integer(retVal.toString());
					} else if ("Date".equalsIgnoreCase(type)){
						DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS Z");
						try {
							retVal = (Date)formatter.parse(retVal.toString());
						} catch (ParseException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
		return retVal;
	}
	
	public GenericItemInterface nextItem() {
		if(actualMetric<metricNumber){
			Map<String, Object> workItemFields = workList.get(actualMetric);
			if (workItemFields!=null) {
				actualMetric++;
				return createItem(workItemFields);
			}
		}
		return null;
	}

	public boolean hasNext() {
		if (actualMetric<metricNumber) return true;
		else return false;

	}

	private GenericItemInterface createItem(Map<String, Object> workitemFields){
		GenericItem item =  new GenericItem();
		// mappa dei campi da valorizzare per l'interfaccia
		Iterator<String> iter = workitemFields.keySet().iterator();
		while (iter.hasNext()) {
			String key = iter.next();
			Object value = workitemFields.get(key);
			item.setValue(key, value);
		}
		return item;
	}
}
